'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tokenizer$1 = _interopDefault(require('tokenizer'));
var highlighter$1 = _interopDefault(require('highlighter'));

/**
 * This is a separate tokenizer to make sure tokenizing of object_filter & highlighter are the same.
 *
 * The tokenizer takes a string, normalizes it and splits it up into searchable tokens.
 */
function tokenize(query) {
  return query.trim().toLowerCase().split(' ');
}
var tokenizer = {
  tokenize: tokenize
};

/**
 * This module provides utility functions to make highlighting text much simpler.
 */

function findPositionOftoken(text, token, startAt) {
  var index = text.indexOf(token, startAt);
  if (index < 0) return null;
  return {
    start: index,
    length: token.length
  };
}

function findPositionsOfToken(text, token) {
  if (token.length === 0) return [];
  var start = 0;
  var results = [];
  var match = findPositionOftoken(text, token, start);

  while (match != null) {
    results.push({
      start: match.start,
      length: token.length
    });
    start = match.start + token.length;
    match = findPositionOftoken(text, token, start);
  }

  return results;
}

function findMatches(text, query) {
  var tokens = tokenizer.tokenize(query);
  return findPositionsOfAllTokens(text, tokens);
}

function findPositionsOfAllTokens(text, tokens) {
  var lowerCaseText = text.toLowerCase();
  return tokens.reduce(function (result, token) {
    var positionsOfCurrenToken = findPositionsOfToken(lowerCaseText, token);
    result.push.apply(result, positionsOfCurrenToken);
    return result;
  }, []);
}

function createHighlightListFromMatches(text, matches) {
  var highlightList = [];
  var start = 0;
  matches.forEach(function (match) {
    var prefix = text.slice(start, match.start);

    if (prefix.length > 0) {
      highlightList.push({
        value: prefix,
        highlighted: false
      });
    }

    var matchedText = text.slice(match.start, match.start + match.length);
    highlightList.push({
      value: matchedText,
      highlighted: true
    });
    start = match.start + match.length;
  });
  var remainder = text.slice(start);

  if (remainder.length > 0) {
    highlightList.push({
      value: remainder,
      highlighted: false
    });
  }

  return highlightList;
}
/**
 * Creates a highlightList from text and a query
 */


function createHighlightList(text, query) {
  var tokens = tokenizer.tokenize(query);
  return createHighlightListFromTokens(text, tokens);
}
function createHighlightListFromTokens(text, tokens) {
  var matches = findPositionsOfAllTokens(text, tokens);
  var sortedMatches = matches.sort(function (a, b) {
    return a.start - b.start;
  });
  var res = createHighlightListFromMatches(text, sortedMatches);
  return res;
}
/**
 * Test whether the string contains part of a query
 */

function stringContainsAnyOfQuery(text, query) {
  var tokens = tokenizer.tokenize(query);
  return stringContainsAnyTokens(text, tokens);
}
function stringContainsAnyTokens(text, tokens) {
  var lowerCaseText = text.toLowerCase();
  return tokens.some(function (token) {
    return lowerCaseText.includes(token);
  });
}
var highlighter = {
  createHighlightList: createHighlightList,
  createHighlightListFromTokens: createHighlightListFromTokens,
  stringContainsAnyOfQuery: stringContainsAnyOfQuery,
  stringContainsAnyTokens: stringContainsAnyTokens,
  findMatches: findMatches
};

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o) {
  var i = 0;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (o = _unsupportedIterableToArray(o))) return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  i = o[Symbol.iterator]();
  return i.next.bind(i);
}

function highlightHTMLElement(element, query) {
  var tokens = tokenizer$1.tokenize(query);
  return highlightHTMLElementWithTokens(element, tokens);
}
function highlightHTMLElementWithTokens(element, tokens) {
  var children = element.childNodes;
  children.forEach(function (child) {
    if (child.nodeType === Node.ELEMENT_NODE) {
      // @ts-ignore
      highlightHTMLElementWithTokens(child, tokens);
    }

    if (child.nodeType === Node.TEXT_NODE) {
      // @ts-ignore
      highlightTextNode(child, tokens);
    }
  });
}

function highlightTextNode(textNode, tokens) {
  var text = textNode.data;
  var highlightList = highlighter$1.createHighlightListFromTokens(text, tokens);
  if (highlightList.length === 1) return;
  var parent = textNode.parentElement;

  for (var _iterator = _createForOfIteratorHelperLoose(highlightList), _step; !(_step = _iterator()).done;) {
    var highlightItem = _step.value;

    if (highlightItem.highlighted) {
      var markedNode = createMarkedNode(text);
      parent === null || parent === void 0 ? void 0 : parent.insertBefore(textNode, markedNode);
    } else {
      var _textNode = createTextNode(text);

      parent === null || parent === void 0 ? void 0 : parent.insertBefore(_textNode, _textNode);
    }
  }
}

function createTextNode(text) {
  return document.createTextNode(text);
}

function createMarkedNode(text) {
  var markElement = document.createElement('mark');
  var textNode = createTextNode(text);
  markElement.appendChild(textNode);
  return markElement;
}

var DOMHighlighter = {
  highlightHTMLElement: highlightHTMLElement,
  highlightHTMLElementWithTokens: highlightHTMLElementWithTokens
};

var createHighlightList$1 = highlighter.createHighlightList;
var createHighlightListFromTokens$1 = highlighter.createHighlightListFromTokens;
var tokenize$1 = tokenizer.tokenize;
var highlightElement = DOMHighlighter.highlightHTMLElement;
var highlightElementWithTokens = DOMHighlighter.highlightHTMLElementWithTokens;
var index = {
  createHighlightList: highlighter.createHighlightList,
  createHighlightListFromTokens: highlighter.createHighlightListFromTokens,
  tokenize: tokenizer.tokenize,
  stringContainsAnyOfQuery: highlighter.stringContainsAnyOfQuery,
  stringContainsAnyOfTokens: highlighter.stringContainsAnyTokens,
  highlightElement: DOMHighlighter.highlightHTMLElement,
  highlightElementWithTokens: DOMHighlighter.highlightHTMLElementWithTokens
};

exports.createHighlightList = createHighlightList$1;
exports.createHighlightListFromTokens = createHighlightListFromTokens$1;
exports.default = index;
exports.highlightElement = highlightElement;
exports.highlightElementWithTokens = highlightElementWithTokens;
exports.tokenize = tokenize$1;
//# sourceMappingURL=fluo.cjs.development.js.map
